//-----------------------------------------
// Cruise Control -- LScript Item Animation
// Arnie Cachelin
//  Move item with constant speed. The direction of motion is the direction at time 0,
// so there must be at least 2 keys in the object motion, preferably linear. 

@version 2.1
@warnings
@script motion

// global values go here
speed = 1.0;
units = 0;
mag = 1.0;
stFrame = 0.0;
endFrame = 0.01;

create
{
}

destroy
{
}

process: ma, frame, time
{
     del = <0,0,0>;
     pi = <0,0,0>;  
     pf = <0,0,0>;



    scene = Scene(); 

    obj = ma.objID;    

    // Get positions at 2 keys used to make linear trajectory
    t = stFrame;
    t /= (scene.fps);
    pi = ma.get(POSITION, t );
    t = endFrame;
    t /= (scene.fps);
    pf = ma.get(POSITION,t);  

    del = pf - pi;   // find line between 
                      
    mag = vmag(del);
    if(mag>0.0)
        mag = speed*time/mag;
    del.x *= mag; // scale line between to speed
    del.y *= mag;
    del.z *= mag;
    pf = pi + del; // add new displacement to initial position
    ma.set(POSITION,pf);
}

load: what,io
{
    if(what == SCENEMODE)   // processing an ASCII scene file
    {
    }
}

save: what,io
{
    if(what == SCENEMODE)
    {
    }
}

options
{
    if(!reqbegin("Cruise Control"))
        return;

    reqsize(372,114);

    c1 = ctlnumber("Speed",1.0);
    ctlposition(c1,28,24);

    c2 = ctlpopup(" ",1,@"m/s","km/Hour","Miles/Hour","Knots"@);
    ctlposition(c2,142,24);

    if(reqpost())
    {
        v1 = getvalue(c1);
        units = getvalue(c2);
        if(units==0)
            speed = v1; // m/s
        else if(units==2)
            speed = v1*0.447; // mi/h
        else if(units==1)
            speed = v1*0.2778; // km/h
        else if(units==3)
            speed = v1*0.5144; // Knots 
    }

}

